@@ -29,17 +29,22 @@
 #include "xmlrpc-c/server_abyss.h"
 
 
-/*=========================================================================
-**  die_if_fault_occurred
-**=========================================================================
-**  If certain kinds of out-of-memory errors occur during server setup,
-**  we want to quit and print an error.
-*/
 
-static void die_if_fault_occurred(xmlrpc_env *env) {
-    if (env->fault_occurred) {
+struct xmlrpc_server_abyss {
+    TServer       abyssServer;
+    TChanSwitch * chanSwitchP;
+    bool          shutdownEnabled;
+        /* User wants system.shutdown to succeed */
+};
+
+
+
+static void
+dieIfFaultOccurred(xmlrpc_env * const envP) {
+
+    if (envP->fault_occurred) {
         fprintf(stderr, "Unexpected XML-RPC fault: %s (%d)\n",
-                env->fault_string, env->fault_code);
+                envP->fault_string, envP->fault_code);
         exit(1);
     }
 }
@@ -533,7 +538,7 @@ termUriHandler(void * const arg) {
 
 
 static void
-handleXmlrpcReq(URIHandler2 * const this,
+handleXmlrpcReq(void *        const handlerArg,
                 TSession *    const abyssSessionP,
                 abyss_bool *  const handledP) {
 /*----------------------------------------------------------------------------
@@ -547,9 +552,9 @@ handleXmlrpcReq(URIHandler2 * const this,
    Note that failing the request counts as handling it, and not handling
    it does not mean we failed it.
 
-   This is an Abyss HTTP Request handler -- type URIHandler2.
+   This is an Abyss HTTP Request handler -- type handleReqFn3.
 -----------------------------------------------------------------------------*/
-    struct uriHandlerXmlrpc * const uriHandlerXmlrpcP = this->userdata;
+    struct uriHandlerXmlrpc * const uriHandlerXmlrpcP = handlerArg;
 
     const TRequestInfo * requestInfoP;
 
@@ -616,6 +621,10 @@ handleXmlrpcReq(URIHandler2 * const this,
 }
 
 
+/* This doesn't include what the user's method function requires */
+#define HANDLE_XMLRPC_REQ_STACK 1024
+
+
 
 /*=========================================================================
 **  xmlrpc_server_abyss_default_handler
@@ -661,7 +670,6 @@ setHandler(xmlrpc_env *      const envP,
            bool              const chunkResponse) {
     
     struct uriHandlerXmlrpc * uriHandlerXmlrpcP;
-    URIHandler2 uriHandler;
     abyss_bool success;
 
     trace_abyss = getenv("XMLRPC_TRACE_ABYSS");
@@ -671,15 +679,18 @@ setHandler(xmlrpc_env *      const envP,
     uriHandlerXmlrpcP->registryP     = registryP;
     uriHandlerXmlrpcP->uriPath       = strdup(uriPath);
     uriHandlerXmlrpcP->chunkResponse = chunkResponse;
-
-    uriHandler.handleReq2 = handleXmlrpcReq;
-    uriHandler.handleReq1 = NULL;
-    uriHandler.userdata   = uriHandlerXmlrpcP;
-    uriHandler.init       = NULL;
-    uriHandler.term       = &termUriHandler;
-
-    ServerAddHandler2(srvP, &uriHandler, &success);
-
+    
+    {
+        size_t const stackSize = 
+            HANDLE_XMLRPC_REQ_STACK + xmlrpc_registry_max_stackSize(registryP);
+        struct ServerReqHandler3 const handlerDesc = {
+            /* .term               = */ &termUriHandler,
+            /* .handleReq          = */ &handleXmlrpcReq,
+            /* .userdata           = */ uriHandlerXmlrpcP,
+            /* .handleReqStackSize = */ stackSize
+        };
+        ServerAddHandler3(srvP, &handlerDesc, &success);
+    }
     if (!success)
         xmlrpc_faultf(envP, "Abyss failed to register the Xmlrpc-c request "
                       "handler.  ServerAddHandler2() failed.");
@@ -968,7 +979,7 @@ enableShutdownParm(const xmlrpc_server_abyss_parms * const parmsP,
 static xmlrpc_server_shutdown_fn shutdownAbyss;
 
 static void
-shutdownAbyss(xmlrpc_env * const envP,
+shutdownAbyss(xmlrpc_env * const faultP,
               void *       const context,
               const char * const comment ATTR_UNUSED,
               void *       const callInfo ATTR_UNUSED) {
@@ -980,17 +991,21 @@ shutdownAbyss(xmlrpc_env * const envP,
 
    After we return, Abyss will finish up the system.shutdown and any
    other connections that are in progress, then the call to
-   ServerRun() etc. will return.  But Abyss may be stuck waiting for
-   something, such as the next HTTP connection.  In that case, until it
-   gets what it's waiting for, it won't even know it's supposed t shut
-   down.  In particular, a caller of system.shutdown may have to execute
-   one more RPC in order for the shutdown to happen.
------------------------------------------------------------------------------*/
-    TServer * const serverP = context;
+   ServerRun() etc. will return.
 
-    xmlrpc_env_init(envP);
+   *faultP is the result of the shutdown request, not whether we
+   succeeded or failed.  We are not allowed to fail.
+-----------------------------------------------------------------------------*/
+    xmlrpc_server_abyss_t * const serverP = context;
+
+    xmlrpc_env_init(faultP);
     
-    ServerTerminate(serverP);
+    if (!serverP->shutdownEnabled)
+        xmlrpc_env_set_fault_formatted(
+            faultP, XMLRPC_REQUEST_REFUSED_ERROR,
+            "Shutdown by client is disabled on this server.");
+    else
+        ServerTerminate(&serverP->abyssServer);
 }
 
 
@@ -999,13 +1014,6 @@ shutdownAbyss(xmlrpc_env * const envP,
   xmlrpc_server_abyss object methods
 =============================================================================*/
 
-struct xmlrpc_server_abyss {
-    TServer       abyssServer;
-    TChanSwitch * chanSwitchP;
-};
-
-
-
 void
 xmlrpc_server_abyss_create(xmlrpc_env *                      const envP,
                            const xmlrpc_server_abyss_parms * const parmsP,
@@ -1023,9 +1031,9 @@ xmlrpc_server_abyss_create(xmlrpc_env *                      const envP,
             xmlrpc_faultf(envP,
                           "You must specify members at least up through "
                           "'registryP' in the server parameters argument.  "
-                          "That would mean the parameter size would be >= %lu "
+                          "That would mean the parameter size would be >= %u "
                           "but you specified a size of %u",
-                          XMLRPC_APSIZE(registryP), parmSize);
+                          (unsigned)XMLRPC_APSIZE(registryP), parmSize);
         else {
             MALLOCVAR(serverP);
 
@@ -1037,10 +1045,11 @@ xmlrpc_server_abyss_create(xmlrpc_env *                      const envP,
                              &serverP->abyssServer, &serverP->chanSwitchP);
             
                 if (!envP->fault_occurred) {
-                    if (enableShutdownParm(parmsP, parmSize))
-                        xmlrpc_registry_set_shutdown(
-                            parmsP->registryP, &shutdownAbyss,
-                            &serverP->abyssServer);
+                    serverP->shutdownEnabled =
+                        enableShutdownParm(parmsP, parmSize);
+
+                    xmlrpc_registry_set_shutdown(
+                        parmsP->registryP, &shutdownAbyss, serverP);
                 
                     if (envP->fault_occurred)
                         free(serverP);
@@ -1355,9 +1364,9 @@ xmlrpc_server_abyss(xmlrpc_env *                      const envP,
             xmlrpc_faultf(envP,
                           "You must specify members at least up through "
                           "'registryP' in the server parameters argument.  "
-                          "That would mean the parameter size would be >= %lu "
+                          "That would mean the parameter size would be >= %u "
                           "but you specified a size of %u",
-                          XMLRPC_APSIZE(registryP), parmSize);
+                          (unsigned)XMLRPC_APSIZE(registryP), parmSize);
         else {
             if (parmsP->config_file_name)
                 oldHighLevelAbyssRun(envP, parmsP, parmSize);
@@ -1408,7 +1417,7 @@ xmlrpc_server_abyss_init_registry(void) {
 
     xmlrpc_env_init(&env);
     builtin_registryP = xmlrpc_registry_new(&env);
-    die_if_fault_occurred(&env);
+    dieIfFaultOccurred(&env);
     xmlrpc_env_clean(&env);
 
     setHandlers(&globalSrv, "/RPC2", builtin_registryP, false);
@@ -1438,7 +1447,7 @@ xmlrpc_server_abyss_add_method(char *        const method_name,
     xmlrpc_env_init(&env);
     xmlrpc_registry_add_method(&env, builtin_registryP, NULL, method_name,
                                method, user_data);
-    die_if_fault_occurred(&env);
+    dieIfFaultOccurred(&env);
     xmlrpc_env_clean(&env);
 }
 
@@ -1456,7 +1465,7 @@ xmlrpc_server_abyss_add_method_w_doc(char *        const method_name,
     xmlrpc_registry_add_method_w_doc(
         &env, builtin_registryP, NULL, method_name,
         method, user_data, signature, help);
-    die_if_fault_occurred(&env);
+    dieIfFaultOccurred(&env);
     xmlrpc_env_clean(&env);    
 }
 
